export default function (Vue) {
    const { h, ref, watch, toRef, Teleport, reactive, onMounted, onUnmounted, defineComponent, computed, } = Vue;
    return function toVueComponent(HFC, lowercase) {
        HFC.propTypes = HFC.propTypes || {};
        const propKeys = Object.keys(HFC.propTypes.attrs || {});
        const eventKeys = Object.keys(HFC.propTypes.events || {});
        const slotKeys = Object.keys(HFC.propTypes.slots || {});
        const propKeyMap = {};
        const eventKeyMap = {};
        const slotKeyMap = {};
        propKeys.forEach((k) => {
            propKeyMap[lowercase ? k.toLowerCase() : k] = k;
        });
        eventKeys.forEach((k) => {
            eventKeyMap[lowercase
                ? "on" + k[0].toUpperCase() + k.slice(1).toLowerCase()
                : "on" + k[0].toUpperCase() + k.slice(1)] = k;
        });
        slotKeys.forEach((k) => {
            slotKeyMap[lowercase ? k.toLowerCase() : k] = k;
        });
        const observePropKeys = Object.keys(propKeyMap);
        const observeEventKeys = Object.keys(eventKeyMap);
        const observeSlotKeys = Object.keys(slotKeyMap);
        return defineComponent({
            props: observePropKeys,
            setup(vueProps, { slots: vueSlots, attrs: vueAttrs }) {
                const container = ref();
                const events = {};
                if (observeEventKeys.length) {
                    for (let i = 0; i < observeEventKeys.length; i++) {
                        const key = observeEventKeys[i];
                        const event = vueAttrs[key];
                        if (!event)
                            continue;
                        events[eventKeyMap[key]] = event;
                    }
                }
                const slots = {};
                const renderedSlots = reactive({});
                if (observeSlotKeys.length) {
                    for (let i = 0; i < observeSlotKeys.length; i++) {
                        const key = observeSlotKeys[i];
                        const slot = vueSlots[key];
                        if (!slot)
                            continue;
                        slots[slotKeyMap[key]] = function (container, args) {
                            if (!container) {
                                delete renderedSlots[key];
                                return;
                            }
                            const slotItem = renderedSlots[key];
                            if (!slotItem) {
                                renderedSlots[key] = { name: key, container, slot, args };
                                return;
                            }
                            if (slotItem.container !== container) {
                                slotItem.container = container;
                            }
                            slotItem.args = args;
                        };
                    }
                }
                let hfc;
                const attrs = {};
                if (observePropKeys.length) {
                    for (let i = 0; i < observePropKeys.length; i++) {
                        const key = observePropKeys[i];
                        if (!vueProps[key])
                            continue;
                        attrs[propKeyMap[key]] = vueProps[key];
                        watch(toRef(vueProps, key), (curr, prev) => {
                            var _a;
                            attrs[propKeyMap[key]] = curr;
                            (_a = hfc.changed) === null || _a === void 0 ? void 0 : _a.call(hfc, "attr", propKeyMap[key], prev, curr);
                        }, { deep: true });
                    }
                }
                hfc = new HFC({
                    attrs,
                    events,
                    slots,
                });
                onMounted(() => {
                    hfc.connected(container.value);
                });
                onUnmounted(() => {
                    var _a;
                    (_a = hfc.disconnected) === null || _a === void 0 ? void 0 : _a.call(hfc);
                });
                const renderedSlotVnodes = computed(() => {
                    return Object.keys(renderedSlots).map((key) => {
                        const item = renderedSlots[key];
                        return h(Teleport, { to: item.container }, item.slot(item.args));
                    });
                });
                return () => [h("div", { ref: container }), renderedSlotVnodes.value];
            },
        });
    };
}
